% ===== MATLAB codes for Fig. 3 of [a]
% [a]	C.-L. Liu and P. P. Vaidyanathan, 
%		Cramer-Rao Bounds for Coprime and Other Sparse Arrays, which Find More Sources than Sensors,
%		Digital Signal Processing.
%       http://dx.doi.org/10.1016/j.dsp.2016.04.011  
%
% Written by Chun-Lin Liu
% E-mail : cl.liu@caltech.edu
% Website: http://systems.caltech.edu/dsp/students/clliu/CRB.html
% 
% Last revised on July 6, 2016
%

clear; clc; close all;

% Sensor locations
S = [1; 2; 3; 6];
% SNR grid (in dB)
SNRdB_grid = -20 : 10 : 40;
% D = 1, 2, 3, 4, 5
DD_grid = 1 : 5;
% Snapshots
K = 500;


CRB_11 = zeros(length(SNRdB_grid), length(DD_grid));
CRB_11_large_SNR = zeros(length(SNRdB_grid), length(DD_grid));

for ii = 1 : length(SNRdB_grid)
    for jj = 1 : length(DD_grid)
        p = 10^(SNRdB_grid(ii)/10);
        DD = DD_grid(jj);
        delta_theta = 0.90 / DD;
        theta_bar = -0.49 + (0:DD-1).'*delta_theta;
        
        CRB_theta_nested = CRB_Sparse_Arrays_Liu_Vaidyanathan( S, theta_bar, p, K );
        CRB_11(ii, jj) = abs( CRB_theta_nested(1,1) );
        
        CRB_theta_nested_high_SNR = CRB_Sparse_Arrays_Liu_Vaidyanathan_large_SNR( S, theta_bar, p, K );
        CRB_11_large_SNR(ii, jj) = abs( CRB_theta_nested_high_SNR(1,1) );
        
        
    end
end


% Figure 3(a)
figure;
semilogy(...
    SNRdB_grid, CRB_11(:, 1), 'o-', ...
    SNRdB_grid, CRB_11(:, 2), 's-', ...
    SNRdB_grid, CRB_11(:, 3), 'd-', ...
    SNRdB_grid, CRB_11_large_SNR(:, 1), '*--', ...
    SNRdB_grid, CRB_11_large_SNR(:, 2), 'x--', ...
    SNRdB_grid, CRB_11_large_SNR(:, 3), 'p--');
grid on;
xlabel('SNR (dB)', 'interpreter', 'latex')
ylabel('$\mathrm{CRB}(\bar{\theta}_1)$', 'interpreter', 'latex')
leg = legend(...
    '$D = 1$', ...
    '$D = 2$', ...
    '$D = 3$', ...
    '$D = 1$, Asymptotic \quad', ...
    '$D = 2$, Asymptotic \quad', ...
    '$D = 3$, Asymptotic \quad');
set(leg, 'interpreter', 'latex', 'Location', 'NorthEast');

% Figure 3(b)
figure;
semilogy(...
    SNRdB_grid, CRB_11(:, 4), 'h-', ...
    SNRdB_grid, CRB_11(:, 5), 'v-', ...
    SNRdB_grid, CRB_11_large_SNR(:, 4), '*--', ...
    SNRdB_grid, CRB_11_large_SNR(:, 5), 'x--');
grid on;
xlabel('SNR (dB)', 'interpreter', 'latex')
ylabel('$\mathrm{CRB}(\bar{\theta}_1)$', 'interpreter', 'latex')
leg = legend(...
    '$D = 4$', ...
    '$D = 5$', ...
    '$D = 4$, Asymptotic \quad', ...
    '$D = 5$, Asymptotic');
set(leg, 'interpreter', 'latex', 'Location', 'NorthEast');
